/* Neon from https://github.com/voronianski/oceanic-next-color-scheme */

@import url(https://fonts.googleapis.com/css?family=Droid+Sans+Mono)

$fg: #f8f8f8
$bg: #171717
$font-family: "Droid Sans Mono"
$shadow: 8px
$shadow-alpha: .9

$colors: #171717, #d81765, #97d01a, #ffa800, #16b1fb, #ff2491, #0fdcb6, #ebebeb, #38252c, #ff0000, #76b639, #e1a126, #289cd5, #ff2491, #0a9b81, #f8f8f8

@import variables
@import styles
