/* Monokai style based on the original TextMate Theme by Wimer Hazenberg */

@import url(https://fonts.googleapis.com/css?family=Fira+Mono:400,700)

$fg: #f8f8f2
$bg: #272822
$font-family: "Fira Mono"
$font-line-height: 1.05
$colors: #1b1d1e, #c82668, #a6e22e, #fd971f, #52aeca, #a17bd7, #465457, #ccccc6, #505354, #ff5995, #b6e354, #feed6c, #8cedff, #9e6ffe, #899ca1, #f8f8f2


@import variables
@import styles
