/* Zenburn style from http://kippura.org/zenburnpage/ */

@import url(https://fonts.googleapis.com/css?family=Inconsolata:400,700)

$fg: #dcdccc
$bg: #3f3f3f
$font-family: "Inconsolata"
$font-line-height: 1.1
$colors: #3f3f3f, #cc9393, #7f9f7f, #e3ceab, #dfaf8f, #cc9393, #8cd0d3, #dcdccc, #3f3f3f, #cc9393, #7f9f7f, #e3ceab, #dfaf8f, #cc9393, #8cd0d3, #dcdccc


@import variables
@import styles
