/* Visibone-alt-2 by Gutterslob from http://terminal.sexy  */
$fg: #f8f8f2
$bg: #282a36
$active-bg: darken(#44475a, 8%)

$colors: #000000, #ff5454, #50fa7a, #f1fa8c, #bd93f9, #ff79c6, #8be9e9, #bbbbbb, #555555, #ff5454, #50fa7a, #f1fa8c, #bd93f9, #ff79c6, #8be8fd, #eeeeec

@import variables
@import styles
