/* Oceanic style from https://github.com/voronianski/oceanic-next-color-scheme */

@import url(https://fonts.googleapis.com/css?family=Droid+Sans+Mono)

$fg: #cdd3de
$bg: #1b2b34
$font-family: "Droid Sans Mono"
$font-size: .9em

$colors: #1b2b34, #ec5f67, #99c794, #fac863, #6699cc, #c594c5, #5fb3b3, #a7adba, #4f5b66, #ec5f67, #99c794, #fac863, #6699cc, #c594c5, #5fb3b3, #d8dee9

@import variables
@import styles
